==> crossroads-0.11.tar.gz: crossroads-0.11/src/crossroads.h <==
/* Includes. */

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <math.h>
#include <pwd.h>
#include <setjmp.h>
#include <signal.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <syslog.h>
#include <time.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/file.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/types.h>

/* Types. */
typedef enum {				/* Config parsing related */
    cf_portspec,
    cf_verbosityspec,
    cf_backendspec,
    cf_serverspec,
    cf_dispatchspec,
    cf_revivespec,
    cf_shmkeyspec,
    cf_weightspec,
    cf_decayspec,
    cf_onsuccspec,
    cf_onfailspec,
    cf_sessiontimeoutspec,
    cf_maxclientsspec,
    cf_dumpspec,
    cf_bindspec,
} Conftype;

typedef union {				/* Integer of string value */
    int ival;				/* passed around by the parser */
    char *sval;
} IntOrString;

typedef struct {
    Conftype cf;
    IntOrString v;
} Confset;

typedef struct {
    int n;
    Confset *set;
} Confsets;

typedef enum {				/* Dispatching types */
    ds_roundrobin,
    ds_random,
    ds_bysize,
    ds_byduration,
    ds_byorder,
} Dispatchtype;

typedef struct {			/* Backend description */
    char *name;				/* .. back end identifier */
    int port;				/* .. TCP port */
    char *server;			/* .. server name */
    int verbosity;			/* .. debugging on/off */
    int weight;				/* .. weight in backend selection */
    int decay;				/* .. decay in % */
    char *onsuccess;			/* .. system() on success */
    char *onfailure;			/* .. or on failure */
    char *dumpfile;			/* .. traffic dump file */
} Backend;

typedef struct {			/* Service description */
    char *name;				/* .. service name */
    char *bind;				/* .. address to bind to */
    unsigned port;			/* .. listening port */
    unsigned verbosity;			/* .. message generation */
    Dispatchtype dispatchtype;		/* .. backend selection method */
    unsigned dispatchover;		/* .. selection over # sessions */
    unsigned rev_interval;		/* .. dead backend revival interval */
    unsigned backlog;			/* .. # pending TCP connections */
    unsigned shmkey;			/* .. key for SysV IPC */
    unsigned sessiontimeout;		/* .. # secs for timeout handling */
    unsigned maxclients;		/* .. max # of allowed connections */
    Backend *backend;			/* .. the back ends */
    int nbackend;			/* .. size of backend array */
} Service;

typedef enum {				/* Backend availability */    
    st_available,			/* .. all OK */
    st_unavailable,			/* .. temporarily unavailable */
    st_down,				/* .. permanently unavailable */\
    st_waking,				/* .. scheduled for wake up */
    st_unknown,				/* .. no idea (terminator) */
} Backendavail;

typedef struct {			/* Backend state */
    Backendavail avail;			/* .. availability */
    unsigned long totuses;		/* .. times hit */
    unsigned long failures;		/* .. times failed */
    unsigned long long nbytes;		/* .. transferred bytes */
    unsigned long avg_nbytes;		/* .. averaged over # sessions */
    unsigned long long nsec;		/* .. connection durations */
    unsigned long avg_nsec;		/* .. averaged over # sessions */
} Backendstate;

typedef struct {			/* Service reporting (shmem) */
    int pid;				/* .. PID of service daemon */
    int rev_pid;			/* .. PID of revival handler */
    int nclients;			/* .. active clients */
    Backendstate			/* .. states of the back ends */
        backendstate[MAX_BACKEND];
} Servicereport;

typedef enum {				/* Stage of the program: */
    stage_main,				/* .. commandline */
    stage_waiting,			/* .. waiting for connections */
    stage_serving,			/* .. servicing a session */
    stage_retrying,			/* .. waking up a backend */
} Programstage;

/* Globals. */
#ifndef EXTERN
#define EXTERN extern
#endif

EXTERN Service *activeservice;		/* target service of a daemon */
EXTERN char *config_file;		/* config to parse */
EXTERN int current_backend;		/* of a given service */
EXTERN int daemonized;			/* are we forked off yet */
EXTERN int flag_foreground;		/* flag: don't daemonize */
EXTERN int flag_verbose;		/* flag: verbosity */
EXTERN int iflag_present;		/* flag: -i present */
EXTERN int interrupted;			/* got a signal? */
EXTERN char *laststring;		/* semantic lexer value */
EXTERN int nservice;			/* # service descriptions */
EXTERN int org_argc;			/* original argc */
EXTERN char **org_argv;			/* and original argv */
EXTERN Programstage program_stage;	/* stage of the program */
EXTERN int relevant_sigs[];		/* relevant signals */
EXTERN int semid;			/* semaphore ID */
EXTERN Service *service;		/* service descriptions */
EXTERN Servicereport *servicereport;	/* reporter in shared mem */
EXTERN char *state_to_string_map[];	/* backend states as strings */
EXTERN int uid;				/* UID to assume */
EXTERN FILE *yyin;			/* config file handle */
EXTERN int yylineno;			/* input line number */
EXTERN char *yyerrmsg;			/* parsing error msg */
EXTERN char *yytext;			/* lexical buffer */

/* Functions. */
extern void alloc_reporter (Service *active, int first);
extern int backend_available (void);
extern int backend_connect (int is_waking);
extern void choose_backend (void);
extern void dealloc_reporter (Service *s);
extern void dumptraffic (char const *buf, int len, int isclient);
extern void error (char const *fmt, ...);
extern int init_sockaddr (struct sockaddr_in *name,
			   const char *hostname, int port);
extern void interrupt (int sig);
extern void lock_reporter (void);
extern int make_socket (int port, char const *ipaddr);
extern void mark_activity (unsigned long long nbytes,
			   unsigned long long nsec,
			   Backendavail newstate);
extern void msg (char const *fmt, ...);
extern void restart (int ac, char **av);
extern void runservice (void);
extern void sample_conf (int ac, char **av);
extern void serve (int ac, char **av);
extern void servesockets (int clientsock, int backendsock);
extern void set_program_title (char const *fmt, ...);
extern void show_services (int ac, char **av);
extern void show_status (int ac, char **av);
extern char *state_to_string (Backendavail avail);
extern Backendavail string_to_state (char const *str);
extern void stop_daemon (int ac, char **av);
extern char *str_vprintf (char const *fmt, va_list args);
extern void sysrun (char const *cmd);
extern void tcpserve (int sock);
extern void tell_service (int ac, char **av);
extern void unlock_reporter (void);
extern void usage (void);
extern void wakeup_handler (void);
extern void warning (char const *fmt, ...);
extern void writelog (int prio, char const *fmt, va_list args);
extern int yyparse (void);
extern void *xmalloc (unsigned sz);
extern void *xrealloc (void *what, unsigned sz);
extern char *xstrcat (char *what, char const *rest);
extern char *xstrdup (char const *what);

==> crossroads-0.17.tar.gz: crossroads-0.17/src/crossroads.h <==
/* Includes. */

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#ifdef HAVE_MALLOC_H
#include <malloc.h>
#endif
#include <math.h>
#include <pwd.h>
#include <setjmp.h>
#include <signal.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <syslog.h>
#include <time.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/file.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/types.h>

/* flock() macros, incase they aren't present on your system. */
#ifndef LOCK_SH
#define LOCK_SH 1
#endif
#ifndef LOCK_EX
#define LOCK_EX 2
#endif
#ifndef LOCK_NB
#define LOCK_NB 4
#endif
#ifndef LOCK_UN
#define LOCK_UN 8
#endif

/* inet_addr() macros, incase they aren't present on your system. */
#ifndef INADDR_NONE
#define INADDR_NONE ((unsigned long) -1)
#endif

/* Types. */
typedef enum {				/* Config parsing related */
    cf_portspec,
    cf_verbosityspec,
    cf_backendspec,
    cf_serverspec,
    cf_dispatchspec,
    cf_revivespec,
    cf_shmkeyspec,
    cf_weightspec,
    cf_decayspec,
    cf_onsuccspec,
    cf_onfailspec,
    cf_sessiontimeoutspec,
    cf_maxclientsspec,
    cf_dumpspec,
    cf_thruspec,
    cf_bindspec,
} Conftype;

typedef union {				/* Integer of string value */
    int ival;				/* passed around by the parser */
    char *sval;
} IntOrString;

typedef struct {
    Conftype cf;
    IntOrString v;
} Confset;

typedef struct {
    int n;
    Confset *set;
} Confsets;

typedef enum {				/* Dispatching types */
    ds_roundrobin,
    ds_random,
    ds_bysize,
    ds_byduration,
    ds_byorder,
} Dispatchtype;

typedef struct {			/* Backend description */
    char *name;				/* .. back end identifier */
    int port;				/* .. TCP port */
    char *server;			/* .. server name */
    int verbosity;			/* .. debugging on/off */
    int weight;				/* .. weight in backend selection */
    int decay;				/* .. decay in % */
    char *onsuccess;			/* .. system() on success */
    char *onfailure;			/* .. or on failure */
    char *dumpfile;			/* .. traffic dump file */
    char *thruputfile;			/* .. traffic throughput file */
    int maxclients;			/* .. max # of allowed connections */
} Backend;

typedef struct {			/* Service description */
    char *name;				/* .. service name */
    char *bind;				/* .. address to bind to */
    unsigned port;			/* .. listening port */
    unsigned verbosity;			/* .. message generation */
    Dispatchtype dispatchtype;		/* .. backend selection method */
    unsigned dispatchover;		/* .. selection over # sessions */
    unsigned rev_interval;		/* .. dead backend revival interval */
    unsigned backlog;			/* .. # pending TCP connections */
    unsigned shmkey;			/* .. key for SysV IPC */
    unsigned sessiontimeout;		/* .. # secs for timeout handling */
    unsigned maxclients;		/* .. max # of allowed connections */
    Backend *backend;			/* .. the back ends */
    int nbackend;			/* .. size of backend array */
} Service;

typedef enum {				/* Backend availability */    
    st_available,			/* .. all OK */
    st_unavailable,			/* .. temporarily unavailable */
    st_down,				/* .. permanently unavailable */\
    st_waking,				/* .. scheduled for wake up */
    st_unknown,				/* .. no idea (terminator) */
} Backendavail;

typedef struct {			/* Backend state */
    Backendavail avail;			/* .. availability */
    unsigned long totuses;		/* .. times hit */
    unsigned long failures;		/* .. times failed */
    unsigned long long nbytes;		/* .. transferred bytes */
    unsigned long avg_nbytes;		/* .. averaged over # sessions */
    unsigned long long nsec;		/* .. connection durations */
    unsigned long avg_nsec;		/* .. averaged over # sessions */
    unsigned nclients;			/* .. active clients */
} Backendstate;

typedef struct {			/* Service reporting (shmem) */
    int pid;				/* .. PID of service daemon */
    int rev_pid;			/* .. PID of revival handler */
    int nclients;			/* .. active clients */
    Backendstate			/* .. states of the back ends */
        backendstate[MAX_BACKEND];
} Servicereport;

typedef enum {				/* Stage of the program: */
    stage_main,				/* .. commandline */
    stage_waiting,			/* .. waiting for connections */
    stage_serving,			/* .. servicing a session */
    stage_retrying,			/* .. waking up a backend */
} Programstage;

/* Globals. */
#ifndef EXTERN
#define EXTERN extern
#endif

EXTERN Service *activeservice;		/* target service of a daemon */
EXTERN char *config_file;		/* config to parse */
EXTERN int current_backend;		/* of a given service */
EXTERN int daemonized;			/* are we forked off yet */
EXTERN int flag_foreground;		/* flag: don't daemonize */
EXTERN int flag_verbose;		/* flag: verbosity */
EXTERN int iflag_present;		/* flag: -i present */
EXTERN int interrupted;			/* got a signal? */
EXTERN char *laststring;		/* semantic lexer value */
EXTERN int nservice;			/* # service descriptions */
EXTERN int org_argc;			/* original argc */
EXTERN char **org_argv;			/* and original argv */
EXTERN Programstage program_stage;	/* stage of the program */
EXTERN int relevant_sigs[];		/* relevant signals */
EXTERN int semid;			/* semaphore ID */
EXTERN Service *service;		/* service descriptions */
EXTERN Servicereport *servicereport;	/* reporter in shared mem */
EXTERN char *state_to_string_map[];	/* backend states as strings */
EXTERN int uid;				/* UID to assume */
EXTERN FILE *yyin;			/* config file handle */
EXTERN int yylineno;			/* input line number */
EXTERN char *yyerrmsg;			/* parsing error msg */
EXTERN char *yytext;			/* lexical buffer */

/* Functions. */
extern void alloc_reporter (Service *active, int first);
extern int backend_available (void);
extern int backend_connect (int is_waking);
extern void choose_backend (void);
extern void dealloc_reporter (Service *s);
extern void error (char const *fmt, ...);
extern int init_sockaddr (struct sockaddr_in *name,
			   const char *hostname, int port);
extern void interrupt (int sig);
extern void lock_reporter (void);
extern int make_socket (int port, char const *ipaddr);
extern void mark_activity (unsigned long long nbytes,
			   unsigned long long nsec,
			   Backendavail newstate);
extern void msg (char const *fmt, ...);
extern void restart (int ac, char **av);
extern void runservice (void);
extern void sample_conf (int ac, char **av);
extern void serve (int ac, char **av);
extern void servesockets (int clientsock, int backendsock);
extern void set_program_title (char const *fmt, ...);
extern void show_services (int ac, char **av);
extern void show_status (int ac, char **av);
extern char *state_to_string (Backendavail avail);
extern Backendavail string_to_state (char const *str);
extern void stop_daemon (int ac, char **av);
extern char *str_vprintf (char const *fmt, va_list args);
extern void sysrun (char const *cmd);
extern void tcpserve (int sock);
extern void tell_service (int ac, char **av);
extern void thruputlog (char const *buf, int len, int isclient);
extern void trafficlog (char const *buf, int len, int isclient);
extern void unlock_reporter (void);
extern void usage (void);
extern void wakeup_handler (void);
extern void warning (char const *fmt, ...);
extern void writelog (int prio, char const *fmt, va_list args);
extern int yyparse (void);
extern void *xmalloc (unsigned sz);
extern void *xrealloc (void *what, unsigned sz);
extern char *xstrcat (char *what, char const *rest);
extern char *xstrdup (char const *what);
